// Arduino Nano VLF metal Detector
// Nano #1 with TX sinewave output
// 8.75kHz for Garrett ACE coil

// Pin assignments
byte audioPin = 11;      // Audio tone

// Program variables
float deg, rad;          // Degrees and radians
float tx;                // TX value for each data point
byte dacInput[240];      // R-2R
byte syncState = LOW;    // State of SYNC pulse (HIGH or LOW)

// Power supply sync pulse
void sync() {
  if (syncState == LOW) {
    PORTB |= B00000100;  // If SYNC state is LOW, then toggle B2 HIGH
    syncState = HIGH;    // Save current SYNC state
  } else {
    PORTB &= B11111011;  // If SYNC state is HIGH, then toggle B2 LOW
    syncState = LOW;     // Save current SYNC state
  }
}

// GB sample trigger
void gbTrigger() {       // Output GB trigger pulse
  PORTB &= B11111110;    // Set B0 LOW
  PORTB |= B00000001;    // Set B0 HIGH
}

// DISC sample trigger   
void discTrigger() {     // Output DISC trigger pulse
  PORTB &= B11111101;    // Set B1 LOW
  PORTB |= B00000010;    // Set B1 HIGH
}

void setup() {
  // Calculate DAC input values
  deg = 0;                                            // Start with zero degrees
  for (int i = 0; i <= 239; i++) {                    // Sine wave will consist of 240 steps
    rad = deg / 360 * TWO_PI;                         // Convert degrees to radians
    tx = sin(rad) * 2.5 + 2.5;                        // Scale and offset
    // 5V divide by 255 = 0.0196
    // R/2R ladder DAC resolution is 8-bits (19.6mV)
    dacInput[i] = tx / 0.0196;                        // Save value for current step
    deg += 1.5;                                       // Increment by 1.5 degrees
  }
  for (int i = 0; i <= 11; i++) {                     // Set D0 to D11 as outputs
    pinMode(i, OUTPUT);
  }
  for (int i = 0; i <= 10000; i++) {                  // Drive SYNC pulse for 1 second
    sync();
    delayMicroseconds(100);
  }
  noInterrupts();                                     // Disable interrupts
  analogWrite(audioPin, 127);                         // Set audioPin with 50% duty cycle PWM
}

void loop() {
  sync();                                             // Sync voltage converters                                 
  discTrigger();                                      // DISC sample trigger
  for (int i = 0; i <= 119; i++) {                    // Apply positive half of sine wave to DAC inputs
    PORTD = dacInput[i];
  }
  gbTrigger();                                        // GB sample trigger
  for (int i = 120; i <= 239; i++) {                  // Apply negative half of sine wave to DAC inputs
    PORTD = dacInput[i];
  } 
}
